--------------------------------------------------------------------------------
-- Project P8  - CSD 
-- Dedicated processors 
-- https://digsys.upc.edu/csd/P08/P8.html
---------------------------------------------------
-- Example of adder + accumulator circuit 
-- Top entity
-- Strategy: plan C2 using components and signals
-- Tutorial project location:
-- https://digsys.upc.edu/csd/P08/Add_Accum/Add_Accum.html
--------------------------------------------------------------------------------

LIBRARY IEEE;
   USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY Add_Accum_BCD_2digit	IS  
	PORT(	
		CD_L			: 	IN  std_logic;
		OSC_CLK_in		: 	IN  std_logic;
		START_L, STOP_L	: 	IN  std_logic;
		Din				: 	IN 	STD_LOGIC_VECTOR(3 DOWNTO 0);	 		
		LT_L, LU_L		: 	OUT STD_LOGIC_VECTOR(6 DOWNTO 0);	  
		DONE			: 	OUT STD_LOGIC;	-- status flag indicator   
		
		CLKout_L		: 	OUT STD_LOGIC;	-- to monitor the 1 Hz CLK
		LED_BAR			: 	OUT STD_LOGIC_VECTOR(8 DOWNTO 0) -- not used LED
		
	);
END Add_Accum_BCD_2digit ;

ARCHITECTURE schematic OF Add_Accum_BCD_2digit  IS  
-- Components

	COMPONENT Datapath IS  
       PORT(
			CD				: 	IN std_logic;
			CLK				: 	IN std_logic;
			CLR, LD			: 	IN std_logic;
			SP				: 	IN std_logic; -- Stop button
			Din				: 	IN STD_LOGIC_VECTOR(3 DOWNTO 0);
			SEGT_L,SEGU_L	: 	OUT STD_LOGIC_VECTOR(6 DOWNTO 0);	  
			ENDF			:	OUT std_logic	-- status flag indicator 
		);
	END COMPONENT;   
	
	COMPONENT Control_unit IS  
 		Port (	
  	    	CLK 	:	IN	STD_LOGIC;
			CD   	:	IN	STD_LOGIC;	
			ST   	:	IN	STD_LOGIC; 	 	-- external control buttons
			ENDF	:	IN	STD_LOGIC; 		-- data-path status signal 
			
			DONE	: 	OUT  STD_LOGIC; 
			CLEAR	: 	OUT  STD_LOGIC;	 
			LOAD	: 	OUT  STD_LOGIC				
		);
	END COMPONENT;        

	COMPONENT CLK_Generator IS  
      PORT(	
			OSC_CLK_in	:	IN	STD_LOGIC;
			CD 			:	IN	STD_LOGIC;
			CE			:	IN	STD_LOGIC;
			CLK_A_SQ	:	OUT	STD_LOGIC	
		);
	END COMPONENT;  


-- Signals for connecting components together (the internal wires)
	SIGNAL CLK, CLR, LD, STOP, ENDF, START, CD :	STD_LOGIC; 

BEGIN  
-- Instantiation of components

	Chip2	: 	Datapath
	PORT MAP (	   	
-- From component name 	=> 	to signal or port name
		CLK				=> CLK,
		CD				=> CD,
		CLR				=> CLR,
		LD				=> LD,
		SP				=> STOP,
		Din				=> Din,		
		ENDF			=> ENDF,		
		SEGT_L			=> LT_L, -- 7-segment LED for tens digit
		SEGU_L			=> LU_L -- 7-segment LED for units digit	
	);
		
		
	Chip1	: 	Control_unit
	PORT MAP (	   	
-- From component name 	=> 	to signal or port name
  	    	CLK			=> 	CLK, 	--system CLK
			CD			=> 	CD,
			ST			=> 	START,
			ENDF		=> 	ENDF,
			DONE		=> 	DONE,
			CLEAR		=> 	CLR,
			LOAD		=> 	LD
	);

	Chip3	: 	CLK_Generator
	PORT MAP (	   	
-- From component name 	=> 	to signal or port name
			OSC_CLK_in	=> 	OSC_CLK_in,
			CD 			=> 	CD,
			CE			=> 	'1',
			CLK_A_SQ	=>	CLK
	);
		

-- connections and logic between components
	 
START 	<= not(START_L);
CD 		<= not(CD_L);
STOP	<= not(STOP_L);

CLKout_L	<= not(CLK); --HEX5 dp active-low
LED_BAR 	<= "000000000"; -- switch off not used LED

	
END schematic ;
