--------------------------------------------------------------------------------
-- Exercise P3 - CSD - Arithmetic circuits and hierarchical multiple-file design
-- http://digsys.upc.edu
---------------------------------------------------
-- An example of a 8-bit full adder (Adder_8bit) using a hierarchical structure
-- This VHDL file is a translation of the Fig. 2 schematic at:
-- https://digsys.upc.edu/csd/P03/lab03/lab3.html#p_Adder_8bit
--------------------------------------------------------------------------------

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

ENTITY Adder_8bit IS

	PORT (
		A,B	: IN 	STD_LOGIC_VECTOR(7 DOWNTO 0); 				
		Cin	: IN 	STD_LOGIC;    
		S	: OUT 	STD_LOGIC_VECTOR (7 DOWNTO 0); -- result
		Cout: OUT  	STD_LOGIC;
		Z	: OUT 	STD_LOGIC	-- Z = 1 when S(7..0) = 0 and  Cout = 0;
	  );

END Adder_8bit;


ARCHITECTURE hierarchical_structure OF Adder_8bit IS

-- The component to be used:
	COMPONENT Adder_4bit IS  
		PORT (
			A,B		: IN 	STD_LOGIC_VECTOR(3 DOWNTO 0); 				
			Cin		: IN 	STD_LOGIC;    
			S		: OUT 	STD_LOGIC_VECTOR (3 DOWNTO 0);
			Cout	: OUT  	STD_LOGIC;
			Z		: OUT 	STD_LOGIC	
		  );
	END COMPONENT;
	
	COMPONENT Adder_1bit IS  
		PORT (
			Ai,Bi, Ci	: IN STD_LOGIC;				
			So, Co		: OUT STD_LOGIC
		  );
	END COMPONENT;
	
-- Signals (wires and cables)
	SIGNAL	C4, C5, C6, C7, Z1, Z2	: STD_LOGIC;
	SIGNAL	Y	: STD_LOGIC_VECTOR(4 downto 0);

BEGIN
-- Instantiation of up to 5 components:
	Chip1	: Adder_4bit
		PORT MAP (
	-- from component name 	=> to signal or port name
					A		=> A(3 downto 0),
					B 		=> B(3 downto 0),
					Cin 	=> Cin,
					Cout	=> C4, 
		 			S	 	=> S(3 downto 0), 			
		  			Z	 	=> Z1							  					  
		);

	Chip2	: Adder_1bit
		PORT MAP (
	-- from component name 	=> to signal or port name
					Ai		=> A(4),
					Bi 		=> B(4),
					Ci 		=> C4,
					So	 	=> Y(0), 
					Co	 	=> C5 					  
		);

	Chip3	: Adder_1bit
		PORT MAP (
	-- from component name 	=> to signal or port name
					Ai		=> A(5),
					Bi 		=> B(5),
					Ci 		=> C5,
					So	 	=> Y(1), 
					Co	 	=> C6 					  
		);

	Chip4	: Adder_1bit
		PORT MAP (
	-- from component name 	=> to signal or port name
					Ai		=> A(6),
					Bi 		=> B(6),
					Ci 		=> C6,
					So	 	=> Y(2), 
					Co	 	=> C7 					  
		);
	Chip5	: Adder_1bit
		PORT MAP (
	-- from component name 	=> to signal or port name
					Ai		=> A(7),
					Bi 		=> B(7),
					Ci 		=> C7,
					So	 	=> Y(3), 
					Co	 	=> Y(4) 					  
		);		

-- Other circuits and equations:
Z 		<= 	not ( Z2 or Y(4) or Y(3) or Y(2) or Y(1) or Y(0));
S(4) 	<=  Y(0);
S(5) 	<=  Y(1);
S(6) 	<=  Y(2);
S(7) 	<=  Y(3);
Cout 	<=  Y(4);
Z2 <= not(Z1);
	   	 		 		   	   
END hierarchical_structure;
