-------------------------------------------------------------------------------
-- An example of a hierarchical 9-bit binary radix-2 to BCD converter 
-- This VHDL is the translation of the schematic in Fig. 4:
-- https://digsys.upc.edu/csd/P03/Bin_BCD_9bit/Bin_BCD_9bit.html
-- 
-- The component DM74185 is our adaptation using plan B of the classic 
-- chip 74185A truth table. Is is described in the following annex:  
-- https://digsys.upc.edu/csd/P02/Bin_BCD_6bitB/bin_BCD_6bit.html#DM74185

---------------------------------------------------
-- Exercise P3  - CSD - Plan C2 using multiple VHDL files 
-- http://digsys.upc.edu
------------------------------------------------------------------------------

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

entity Bin_BCD_9bit is
	port (
		B	: 	in	STD_LOGIC_VECTOR(8 DOWNTO 0); 
		T, U: 	out	STD_LOGIC_VECTOR(3 DOWNTO 0);
		H	: 	out	STD_LOGIC_VECTOR(2 DOWNTO 0)
	);
end Bin_BCD_9bit;
	
architecture structural_planC2 of Bin_BCD_9bit is

-- Components to be used:
COMPONENT DM74185 IS  
	port (
		E, D, C, B, A 	: 	in STD_LOGIC;
		Y				: 	out STD_LOGIC_VECTOR(6 DOWNTO 1)
	);
END COMPONENT;

-- Signals (wires and cables) 
	SIGNAL	K	: STD_LOGIC_VECTOR(2 downto 0); -- (to connect unused outputs)
	SIGNAL	C	: STD_LOGIC_VECTOR(5 downto 0); 
	SIGNAL	D	: STD_LOGIC_VECTOR(4 downto 0); 

BEGIN
-- Component instantiation 
	Chip1	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E				=> B(8),
			D 				=> B(7),
			C				=> B(6),
			B 				=> B(5),
			A				=> B(4),
			Y(6 downto 1)	=> C(5 downto 0) 	
		);	

	Chip2	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E				=> C(2),
			D 				=> C(1),
			C				=> C(0),
			B 				=> B(3),
			A				=> B(2),
			Y(6)			=> K(2),
			Y(5 downto 1)	=> D(4 downto 0) 
			
		);	

	Chip3	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E				=> '0',
			D 				=> D(2),
			C				=> D(1),
			B 				=> D(0),
			A				=> B(1),
			Y(6 downto 5)	=> K(1 downto 0),
			Y(4)			=> T(0), 
			Y(3 downto 1)	=> U(3 downto 1) 				
		);		

	Chip4	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E	=> C(5),
			D 	=> C(4),
			C	=> C(3),
			B 	=> D(4),
			A	=> D(3),
			Y(6 downto 4)	=> H,
			Y(3 downto 1)	=> T(3 downto 1)
		);
		
-- Other circuits and equations:
U(0) <= B(0);
	
end structural_planC2;
						