--------------------------------------------------------------------------------
-- Example arithmetic circuit 
-- Cascadable 4-bit comparator for radix 2 numbers (unsigned)
-- Structural approach based on equations, single-file flat plan A
-- UPC - EETAC - CSD 
-- https://digsys.upc.edu/csd/P03/Comp_4bitA/Comp_4bit.html
-- Theory, ideas and similar circuits in P3
--------------------------------------------------------------------------------
LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

ENTITY Comp_4bit IS
		PORT (
			A, B		: IN	STD_LOGIC_VECTOR(3 DOWNTO 0); 
			Gi, Ei, Li	: IN	STD_LOGIC;	
			GT, EQ, LT	: OUT	STD_LOGIC    
			);  
END Comp_4bit;

ARCHITECTURE equations OF Comp_4bit IS

-- Signals: internal wires to connect gates
	SIGNAL	E, G, L : STD_LOGIC_VECTOR (3 DOWNTO 0);	

BEGIN
-- Equal to (EQ) equation:
	E(3) <= not(A(3) xor B(3));
	E(2) <= not(A(2) xor B(2));	
	E(1) <= not(A(1) xor B(1));
	E(0) <= not(A(0) xor B(0));
	EQ <= E(3) and E(2) and E(1) and E(0) and Ei;

-- Greater than (GT) equation:
	G(3) <= A(3) and not(B(3));
	G(2) <= A(2) and not(B(2));
	G(1) <= A(1) and not(B(1));
	G(0) <= A(0) and not(B(0));	

	GT <= G(3) or (E(3) and G(2)) 
			   or (E(3) and E(2) and G(1))
			   or (E(3) and E(2) and E(1) and G(0)) 
			   or (E(3) and E(2) and E(1) and E(0) and Gi) ;

-- Less than (LT) equation:
	L(3) <= not(A(3)) and B(3);
	L(2) <= not(A(2)) and B(2);
	L(1) <= not(A(1)) and B(1);
	L(0) <= not(A(0)) and B(0);	

	LT <= L(3) or (E(3) and L(2)) 
			   or (E(3) and E(2) and L(1))
			   or (E(3) and E(2) and E(1) and L(0)) 
			   or (E(3) and E(2) and E(1) and E(0) and Li) ;
						   
											   
END equations;
