
--------------------------------------------------------------------------------
-- Example arithmetic circuit 
-- 10-bit comparator for radix 2 numbers (unsigned)
-- Structural approach (Plan C2) and cascadable. 
-- UPC - EETAC - CSD 
-- https://digsys.upc.edu/csd/P03/Comp_10bit/Comp_10bit.html
-- Theory, ideas and similar circuits in P3
-- This VHDL translation corresponds with the schematic in Fig.3 attribute
-- https://digsys.upc.edu/csd/P03/Comp_10bit/img_planC2.jpg
--------------------------------------------------------------------------------
LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

ENTITY Comp_8bit IS
		PORT (
			A, B		: IN	STD_LOGIC_VECTOR(7 DOWNTO 0); 
			Gi, Ei, Li	: IN	STD_LOGIC;	
			GT, EQ, LT	: OUT	STD_LOGIC    
			);  
END Comp_8bit;

ARCHITECTURE hierarchical_structure OF Comp_8bit IS

-- Components to be used:
	COMPONENT Comp_4bit IS     -- From P3 tutorial
		PORT (
			A, B		: IN	STD_LOGIC_VECTOR(3 DOWNTO 0); 
			Gi, Ei, Li	: IN	STD_LOGIC;	
			GT, EQ, LT	: OUT	STD_LOGIC    
			);  
		END COMPONENT;

-- Signals, -- The wires to connect the modules:
	SIGNAL	C	:	STD_LOGIC_VECTOR(2 DOWNTO 0); 

BEGIN

-- Instantiating all the Comp_1bit

	Chip0	: Comp_4bit
		PORT MAP (
	-- from component name 	=> to signal or port name
			A		=> A(3 downto 0),
			B		=> B(3 downto 0),
			Gi		=> Gi,
			Ei		=> Ei,			
			Li		=> Li,			
			GT		=> C(2),
			EQ		=> C(1),			
			LT		=> C(0)
		);

	Chip1	: Comp_4bit
		PORT MAP (
	-- from component name 	=> to signal or port name
			A		=> A(7 downto 4),
			B		=> B(7 downto 4),
			Gi		=> C(2),
			Ei		=> C(1),			
			Li		=> C(0),			
			GT		=> GT,
			EQ		=> EQ,			
			LT		=> LT
		);
		
-- Other circuits and equations: not necessary 
			 		   	 		 		   	   
END hierarchical_structure;

		
